/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/

/*
===================================================================

This SHA-2 source code written from and validated with FIPS 180-2.

http://csrc.nist.gov/publications/fips/fips180-2/fips180-2withchangenotice.pdf

Only SHA-2 256bit and SHA-2 512bit have be written.

===================================================================
*/

#include <string.h>

#ifdef _WIN32
typedef unsigned __int64        UINT64;
#else
typedef unsigned long long      UINT64;
#endif
typedef unsigned long           UINT32;
typedef unsigned char           UINT8;

/* 32 bit */
#define M_SHR32(x,c)            ((x)>>(c))
#define M_ROR32(x,c)            (((x)>>(c))|((x)<<(32-(c))))

#define M_LOAD_LE2BE32(px)      ( ( (UINT32)(px)[3]     ) |  \
                                  (((UINT32)(px)[2])<< 8) |  \
                                  (((UINT32)(px)[1])<<16) |  \
                                  (((UINT32)(px)[0])<<24) )
                                  
#define M_LOAD_BE2LE32(px,x)    { (px)[3] = (UINT8)( x     ); \
                                  (px)[2] = (UINT8)((x)>> 8); \
                                  (px)[1] = (UINT8)((x)>>16); \
                                  (px)[0] = (UINT8)((x)>>24); }


/* 64 bits */
#define M_SHR64(x,c)            ((x)>>(c))
#define M_ROR64(x,c)            (((x)>>(c))|((x)<<(64-(c))))

#define M_LOAD_LE2BE64(px)      ( ( (UINT64)(px)[7]     ) | \
                                  (((UINT64)(px)[6])<< 8) | \
                                  (((UINT64)(px)[5])<<16) | \
                                  (((UINT64)(px)[4])<<24) | \
                                  (((UINT64)(px)[3])<<32) | \
                                  (((UINT64)(px)[2])<<40) | \
                                  (((UINT64)(px)[1])<<48) | \
                                  (((UINT64)(px)[0])<<56) )
                                  
#define M_LOAD_BE2LE64(px,x)    { (px)[7] = (UINT8) (x);      \
                                  (px)[6] = (UINT8)((x)>> 8); \
                                  (px)[5] = (UINT8)((x)>>16); \
                                  (px)[4] = (UINT8)((x)>>24); \
                                  (px)[3] = (UINT8)((x)>>32); \
                                  (px)[2] = (UINT8)((x)>>40); \
                                  (px)[1] = (UINT8)((x)>>48); \
                                  (px)[0] = (UINT8)((x)>>56); }

/* 32 & 64 bits */
#define M_CH(x,y,z)             (((x)&(y))^(~(x)&(z)))
#define M_MAJ(x,y,z)            (((x)&(y))^((x)&(z))^((y)&(z)))

/* SHA-2 256 bits */
#define SHA256_BLOCKSIZE        64

#define M_SHA256_EXROR1(x)      (M_ROR32((x),7)^M_ROR32((x),18)^M_SHR32((x),3))
#define M_SHA256_EXROR2(x)      (M_ROR32((x),17)^M_ROR32((x),19)^M_SHR32((x),10))
#define M_SHA256_PROCESST1(x)   (M_ROR32((x),6)^M_ROR32((x),11)^M_ROR32((x),25))
#define M_SHA256_PROCESST2(x)   (M_ROR32((x),2)^M_ROR32((x),13)^M_ROR32((x),22))

typedef struct {
    UINT32 h[8];
    UINT8 bBlockData[SHA256_BLOCKSIZE<<1];
} sha256state;


/* SHA-2 512 bits */
#define SHA512_BLOCKSIZE        128

#define M_SHA512_EXROR1(x)      (M_ROR64((x),1)^M_ROR64((x),8)^M_SHR64((x),7))
#define M_SHA512_EXROR2(x)      (M_ROR64((x),19)^M_ROR64((x),61)^M_SHR64((x),6))
#define M_SHA512_PROCESST1(x)   (M_ROR64((x),14)^M_ROR64((x),18)^M_ROR64((x),41))
#define M_SHA512_PROCESST2(x)   (M_ROR64((x),28)^M_ROR64((x),34)^M_ROR64((x),39))


typedef struct {
    UINT64 h[8];
    UINT8 bBlockData[SHA512_BLOCKSIZE<<1];
} sha512state;




UINT32 sha256_table_k[64] = {
    0x428a2f98, 0x71374491, 0xb5c0fbcf, 0xe9b5dba5, 0x3956c25b, 0x59f111f1, 0x923f82a4, 0xab1c5ed5,
    0xd807aa98, 0x12835b01, 0x243185be, 0x550c7dc3, 0x72be5d74, 0x80deb1fe, 0x9bdc06a7, 0xc19bf174,
    0xe49b69c1, 0xefbe4786, 0x0fc19dc6, 0x240ca1cc, 0x2de92c6f, 0x4a7484aa, 0x5cb0a9dc, 0x76f988da,
    0x983e5152, 0xa831c66d, 0xb00327c8, 0xbf597fc7, 0xc6e00bf3, 0xd5a79147, 0x06ca6351, 0x14292967,
    0x27b70a85, 0x2e1b2138, 0x4d2c6dfc, 0x53380d13, 0x650a7354, 0x766a0abb, 0x81c2c92e, 0x92722c85,
    0xa2bfe8a1, 0xa81a664b, 0xc24b8b70, 0xc76c51a3, 0xd192e819, 0xd6990624, 0xf40e3585, 0x106aa070,
    0x19a4c116, 0x1e376c08, 0x2748774c, 0x34b0bcb5, 0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f, 0x682e6ff3,
    0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208, 0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2 };
    
UINT32 sha256_table_h[8] = {
    0x6a09e667, 0xbb67ae85, 0x3c6ef372, 0xa54ff53a, 0x510e527f, 0x9b05688c, 0x1f83d9ab, 0x5be0cd19 };

UINT64 sha512_table_k[80] = {
#ifdef _WIN32
    0x428a2f98d728ae22, 0x7137449123ef65cd, 0xb5c0fbcfec4d3b2f, 0xe9b5dba58189dbbc, 
    0x3956c25bf348b538, 0x59f111f1b605d019, 0x923f82a4af194f9b, 0xab1c5ed5da6d8118, 
    0xd807aa98a3030242, 0x12835b0145706fbe, 0x243185be4ee4b28c, 0x550c7dc3d5ffb4e2,
    0x72be5d74f27b896f, 0x80deb1fe3b1696b1, 0x9bdc06a725c71235, 0xc19bf174cf692694,
    0xe49b69c19ef14ad2, 0xefbe4786384f25e3, 0x0fc19dc68b8cd5b5, 0x240ca1cc77ac9c65,
    0x2de92c6f592b0275, 0x4a7484aa6ea6e483, 0x5cb0a9dcbd41fbd4, 0x76f988da831153b5,
    0x983e5152ee66dfab, 0xa831c66d2db43210, 0xb00327c898fb213f, 0xbf597fc7beef0ee4,
    0xc6e00bf33da88fc2, 0xd5a79147930aa725, 0x06ca6351e003826f, 0x142929670a0e6e70,
    0x27b70a8546d22ffc, 0x2e1b21385c26c926, 0x4d2c6dfc5ac42aed, 0x53380d139d95b3df,
    0x650a73548baf63de, 0x766a0abb3c77b2a8, 0x81c2c92e47edaee6, 0x92722c851482353b,
    0xa2bfe8a14cf10364, 0xa81a664bbc423001, 0xc24b8b70d0f89791, 0xc76c51a30654be30,
    0xd192e819d6ef5218, 0xd69906245565a910, 0xf40e35855771202a, 0x106aa07032bbd1b8,
    0x19a4c116b8d2d0c8, 0x1e376c085141ab53, 0x2748774cdf8eeb99, 0x34b0bcb5e19b48a8,
    0x391c0cb3c5c95a63, 0x4ed8aa4ae3418acb, 0x5b9cca4f7763e373, 0x682e6ff3d6b2b8a3,
    0x748f82ee5defb2fc, 0x78a5636f43172f60, 0x84c87814a1f0ab72, 0x8cc702081a6439ec,
    0x90befffa23631e28, 0xa4506cebde82bde9, 0xbef9a3f7b2c67915, 0xc67178f2e372532b,
    0xca273eceea26619c, 0xd186b8c721c0c207, 0xeada7dd6cde0eb1e, 0xf57d4f7fee6ed178,
    0x06f067aa72176fba, 0x0a637dc5a2c898a6, 0x113f9804bef90dae, 0x1b710b35131c471b,
    0x28db77f523047d84, 0x32caab7b40c72493, 0x3c9ebe0a15c9bebc, 0x431d67c49c100d4c,
    0x4cc5d4becb3e42b6, 0x597f299cfc657e2a, 0x5fcb6fab3ad6faec, 0x6c44198c4a475817 };
#else
    0x428a2f98d728ae22ULL, 0x7137449123ef65cdULL, 0xb5c0fbcfec4d3b2fULL, 0xe9b5dba58189dbbcULL, 
    0x3956c25bf348b538ULL, 0x59f111f1b605d019ULL, 0x923f82a4af194f9bULL, 0xab1c5ed5da6d8118ULL, 
    0xd807aa98a3030242ULL, 0x12835b0145706fbeULL, 0x243185be4ee4b28cULL, 0x550c7dc3d5ffb4e2ULL,
    0x72be5d74f27b896fULL, 0x80deb1fe3b1696b1ULL, 0x9bdc06a725c71235ULL, 0xc19bf174cf692694ULL,
    0xe49b69c19ef14ad2ULL, 0xefbe4786384f25e3ULL, 0x0fc19dc68b8cd5b5ULL, 0x240ca1cc77ac9c65ULL,
    0x2de92c6f592b0275ULL, 0x4a7484aa6ea6e483ULL, 0x5cb0a9dcbd41fbd4ULL, 0x76f988da831153b5ULL,
    0x983e5152ee66dfabULL, 0xa831c66d2db43210ULL, 0xb00327c898fb213fULL, 0xbf597fc7beef0ee4ULL,
    0xc6e00bf33da88fc2ULL, 0xd5a79147930aa725ULL, 0x06ca6351e003826fULL, 0x142929670a0e6e70ULL,
    0x27b70a8546d22ffcULL, 0x2e1b21385c26c926ULL, 0x4d2c6dfc5ac42aedULL, 0x53380d139d95b3dfULL,
    0x650a73548baf63deULL, 0x766a0abb3c77b2a8ULL, 0x81c2c92e47edaee6ULL, 0x92722c851482353bULL,
    0xa2bfe8a14cf10364ULL, 0xa81a664bbc423001ULL, 0xc24b8b70d0f89791ULL, 0xc76c51a30654be30ULL,
    0xd192e819d6ef5218ULL, 0xd69906245565a910ULL, 0xf40e35855771202aULL, 0x106aa07032bbd1b8ULL,
    0x19a4c116b8d2d0c8ULL, 0x1e376c085141ab53ULL, 0x2748774cdf8eeb99ULL, 0x34b0bcb5e19b48a8ULL,
    0x391c0cb3c5c95a63ULL, 0x4ed8aa4ae3418acbULL, 0x5b9cca4f7763e373ULL, 0x682e6ff3d6b2b8a3ULL,
    0x748f82ee5defb2fcULL, 0x78a5636f43172f60ULL, 0x84c87814a1f0ab72ULL, 0x8cc702081a6439ecULL,
    0x90befffa23631e28ULL, 0xa4506cebde82bde9ULL, 0xbef9a3f7b2c67915ULL, 0xc67178f2e372532bULL,
    0xca273eceea26619cULL, 0xd186b8c721c0c207ULL, 0xeada7dd6cde0eb1eULL, 0xf57d4f7fee6ed178ULL,
    0x06f067aa72176fbaULL, 0x0a637dc5a2c898a6ULL, 0x113f9804bef90daeULL, 0x1b710b35131c471bULL,
    0x28db77f523047d84ULL, 0x32caab7b40c72493ULL, 0x3c9ebe0a15c9bebcULL, 0x431d67c49c100d4cULL,
    0x4cc5d4becb3e42b6ULL, 0x597f299cfc657e2aULL, 0x5fcb6fab3ad6faecULL, 0x6c44198c4a475817ULL };
#endif

UINT64 sha512_table_h[8] = {
#ifdef _WIN32
    0x6a09e667f3bcc908, 0xbb67ae8584caa73b, 0x3c6ef372fe94f82b, 0xa54ff53a5f1d36f1,
    0x510e527fade682d1, 0x9b05688c2b3e6c1f, 0x1f83d9abfb41bd6b, 0x5be0cd19137e2179 };
#else
    0x6a09e667f3bcc908ULL, 0xbb67ae8584caa73bULL, 0x3c6ef372fe94f82bULL, 0xa54ff53a5f1d36f1ULL,
    0x510e527fade682d1ULL, 0x9b05688c2b3e6c1fULL, 0x1f83d9abfb41bd6bULL, 0x5be0cd19137e2179ULL };
#endif


















void sha256_process ( sha256state * pState, UINT8 * pData, UINT32 ui32BlockCount )
{
    UINT32 w[64];
    UINT32 temp[8];
    UINT32 t1, t2;
    UINT8 *pSubBlock;
    long i, j;
    
    for (i=0;i<(long)ui32BlockCount;i++)
    {
        //Bloc into 512bit
        pSubBlock=pData+(i<<6);
        
        //Load Little Endian to Big Endian
        for (j=0;j<16;j++)
            w[j] = M_LOAD_LE2BE32(&(pSubBlock[j<<2]));
        
        //Extend    
        for (j=16;j<64;j++)
            w[j] = w[j-16] + M_SHA256_EXROR1(w[j-15]) + w[j-7] + M_SHA256_EXROR2(w[j-2]);
        
        //Init hash value    
        for (j=0;j<8;j++)
            temp[j] = pState->h[j];

        //Main loop        
        for (j=0;j<64;j++)
        {
            t2 = M_SHA256_PROCESST2(temp[0]) + M_MAJ(temp[0],temp[1],temp[2]);
            t1 = M_SHA256_PROCESST1(temp[4]) + M_CH(temp[4],temp[5],temp[6]) + temp[7] + w[j] + sha256_table_k[j];
            
            temp[7] = temp[6];
            temp[6] = temp[5];
            temp[5] = temp[4];
            temp[4] = temp[3] + t1;
            temp[3] = temp[2];
            temp[2] = temp[1];
            temp[1] = temp[0];
            temp[0] = t1 + t2;
        }
        
        //Add chunk hash to result
        for (j=0;j<8;j++)
            pState->h[j] += temp[j];
    }
}


void sha256 ( unsigned char * pData, unsigned long ui32DataByteCount, unsigned char * pHash )
{
    sha256state State;
    long i;    
    long lAvailableByteCount;
    long lBlockCount;
    long lLoop;
    UINT8 * pDataProcess;
    UINT64 ui64Complement;
    
    //Initialisation
    memset ( &State, 0, sizeof(sha256state) );
    for (i=0;i<8;i++)
        State.h[i] = sha256_table_h[i];       
        
    //Process        
    lAvailableByteCount = (long)ui32DataByteCount;
    pDataProcess = pData;
    lLoop = 1;
    do {
        if (lAvailableByteCount>SHA256_BLOCKSIZE)
        {
            lBlockCount = lAvailableByteCount / SHA256_BLOCKSIZE;
            sha256_process ( &State, pDataProcess, lBlockCount );
            pDataProcess        += (lBlockCount*SHA256_BLOCKSIZE);
            lAvailableByteCount -= (lBlockCount*SHA256_BLOCKSIZE);
        }
        else
        {
            memset ( State.bBlockData+lAvailableByteCount, 0, (SHA256_BLOCKSIZE<<1)-lAvailableByteCount );
            memcpy ( State.bBlockData, pDataProcess, lAvailableByteCount );
            
            //Add the bit 1
            State.bBlockData[lAvailableByteCount] = 0x80;

            //1 or 2 block more ?            
            if ((lAvailableByteCount+9)<=SHA256_BLOCKSIZE) lBlockCount = 1;
            else lBlockCount = 2;
            
            //Add the complement
            ui64Complement = ((UINT64)ui32DataByteCount)<<3;
            M_LOAD_BE2LE64 ( State.bBlockData+(lBlockCount*SHA256_BLOCKSIZE)-8, ui64Complement );
            
            //Final process            
            sha256_process ( &State, State.bBlockData, lBlockCount );
            lAvailableByteCount = 0;
            lLoop = 0;
        }
    } while (lLoop);
    
    //Big endian to little endian    
    for (i = 0 ; i < 8; i++)
        M_LOAD_BE2LE32 ( &pHash[i<<2], State.h[i] );
}

void sha512_process ( sha512state * pState, UINT8 * pData, UINT32 ui32BlockCount )
{
    UINT64 w[80];
    UINT64 temp[8];
    UINT64 t1, t2;
    UINT8 *pSubBlock;
    long i, j;
    
    for (i=0;i<(long)ui32BlockCount;i++)
    {
        //Bloc into 1024bit
        pSubBlock=pData+(i<<7);
        
        //Load Little Endian to Big Endian
        for (j=0;j<16;j++)
            w[j] = M_LOAD_LE2BE64(&(pSubBlock[j<<3]));
        
        //Extend    
        for (j=16;j<80;j++)
            w[j] = w[j-16] + M_SHA512_EXROR1(w[j-15]) + w[j-7] + M_SHA512_EXROR2(w[j-2]);
        
        //Init hash value    
        for (j=0;j<8;j++)
            temp[j] = pState->h[j];

        //Main loop        
        for (j=0;j<80;j++)
        {
            t2 = M_SHA512_PROCESST2(temp[0]) + M_MAJ(temp[0],temp[1],temp[2]);
            t1 = M_SHA512_PROCESST1(temp[4]) + M_CH(temp[4],temp[5],temp[6]) + temp[7] + w[j] + sha512_table_k[j];
            
            temp[7] = temp[6];
            temp[6] = temp[5];
            temp[5] = temp[4];
            temp[4] = temp[3] + t1;
            temp[3] = temp[2];
            temp[2] = temp[1];
            temp[1] = temp[0];
            temp[0] = t1 + t2;
        }
        
        //Add chunk hash to result
        for (j=0;j<8;j++)
            pState->h[j] += temp[j];
    }
}


void sha512 ( unsigned char * pData, unsigned long ui32DataByteCount, unsigned char * pHash )
{
    sha512state State;
    long i;    
    long lAvailableByteCount;
    long lBlockCount;
    long lLoop;
    UINT8 * pDataProcess;
    UINT64 ui64Complement;
    
    //Initialisation
    memset ( &State, 0, sizeof(sha512state) );
    for (i=0;i<8;i++)
        State.h[i] = sha512_table_h[i];       
        
    //Process        
    lAvailableByteCount = (long)ui32DataByteCount;
    pDataProcess = pData;
    lLoop = 1;
    do {
        if (lAvailableByteCount>SHA512_BLOCKSIZE)
        {
            lBlockCount = lAvailableByteCount / SHA512_BLOCKSIZE;
            sha512_process ( &State, pDataProcess, lBlockCount );
            pDataProcess        += (lBlockCount*SHA512_BLOCKSIZE);
            lAvailableByteCount -= (lBlockCount*SHA512_BLOCKSIZE);
        }
        else
        {
            memset ( State.bBlockData+lAvailableByteCount, 0, (SHA512_BLOCKSIZE<<1)-lAvailableByteCount );
            memcpy ( State.bBlockData, pDataProcess, lAvailableByteCount );
            
            //Add the bit 1
            State.bBlockData[lAvailableByteCount] = 0x80;

            //1 or 2 block more ?            
            if ((lAvailableByteCount+17)<=SHA512_BLOCKSIZE) lBlockCount = 1;
            else lBlockCount = 2;
            
            //Add the complement
            ui64Complement = ((UINT64)ui32DataByteCount)<<3;
            M_LOAD_BE2LE64 ( State.bBlockData+(lBlockCount*SHA512_BLOCKSIZE)-8, ui64Complement );
            
            //Final process            
            sha512_process ( &State, State.bBlockData, lBlockCount );
            lAvailableByteCount = 0;
            lLoop = 0;
        }
    } while (lLoop);
    
    //Big endian to little endian    
    for (i = 0 ; i < 8; i++)
        M_LOAD_BE2LE64 ( &pHash[i<<3], State.h[i] );
}





#ifdef FIPS180_2_TEST

#include <stdio.h>
#include "sha2.h"

#define SHA_256_A   "ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad"
#define SHA_256_B   "248d6a61d20638b8e5c026930c3e6039a33ce45964ff2167f6ecedd419db06c1"
#define SHA_256_C   "cdc76e5c9914fb9281a1c7e284d73e67f1809a48a497200e046d39ccc7112cd0"

#define SHA_512_A   "ddaf35a193617abacc417349ae20413112e6fa4e89a97ea20a9eeee64b55d39a2192992a274fc1a836ba3c23a3feebbd454d4423643ce80e2a9ac94fa54ca49f"
#define SHA_512_B   "8e959b75dae313da8cf4f72814fc143f8f7779c6eb9f7fa17299aeadb6889018501d289e4900f7e4331b99dec4b5433ac7d329eeb6dd26545e96e55b874be909"
#define SHA_512_C   "e718483d0ce769644e2e42c7bc15b4638e1f98b13b2044285632a803afa973ebde0ff244877ea60a4cb0432ce577c31beb009c5c2c49aa2e4eadb217ad8cc09b"


void sha2_test_hash2ascii ( char * psHash, char * pszHashAscii, long lLen )
{
    char szTmp[16];
    long i;
    pszHashAscii[0] = 0;
    
    for (i=0;i<lLen;i++)
    {
        sprintf ( szTmp, "%02x", (unsigned char)(psHash[i]) );
        strcat ( pszHashAscii, szTmp );
    }
}

void sha2_test ( void )
{
    char szTest_A[256];
    char szTest_B[256];
    char * szTest_C;
    char sHash[256], szHashAscii[512];
    
    
    strcpy ( szTest_A, "abc" );
    szTest_C = (char *)malloc ( 1000001 );
    memset ( szTest_C, 'a', 1000000 );
    szTest_C[1000000] = 0;
    
    //SHA-2 256bit
    
    //Special Multi block message SHA-256
    strcpy ( szTest_B, "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq" );
    
    printf ( "\nSHA-256 : \n" );
    
    printf ( "Test A 'One block message' :\n" );
    memset ( sHash, 0, 256 );
    sha256 ( szTest_A, strlen(szTest_A), sHash );
    sha2_test_hash2ascii ( sHash, szHashAscii, SHA256_HASHSIZE );
    if (strcmp(szHashAscii,SHA_256_A)==0) printf ( "OK\n" );
    else printf ( "Error : %s\n", szHashAscii ); 

    printf ( "Test B 'Multi block message' :\n" );
    memset ( sHash, 0, 256 );
    sha256 ( szTest_B, strlen(szTest_B), sHash );
    sha2_test_hash2ascii ( sHash, szHashAscii, SHA256_HASHSIZE );
    if (strcmp(szHashAscii,SHA_256_B)==0) printf ( "OK\n" );
    else printf ( "Error : %s\n", szHashAscii ); 

    printf ( "Test C 'Long block message' :\n" );
    memset ( sHash, 0, 256 );
    sha256 ( szTest_C, strlen(szTest_C), sHash );
    sha2_test_hash2ascii ( sHash, szHashAscii, SHA256_HASHSIZE );
    if (strcmp(szHashAscii,SHA_256_C)==0) printf ( "OK\n" );
    else printf ( "Error : %s\n", szHashAscii ); 


    
    //SHA-2 512bit
    
    //Special Multi block message for SHA-512
    strcpy ( szTest_B, "abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmnhijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu" );

    printf ( "\nSHA-512 : \n" );
    
    printf ( "Test A 'One block message' :\n" );
    memset ( sHash, 0, 256 );
    sha512 ( szTest_A, strlen(szTest_A), sHash );
    sha2_test_hash2ascii ( sHash, szHashAscii, SHA512_HASHSIZE );
    if (strcmp(szHashAscii,SHA_512_A)==0) printf ( "OK\n" );
    else printf ( "Error : %s\n", szHashAscii ); 

    printf ( "Test B 'Multi block message' :\n" );
    memset ( sHash, 0, 256 );
    sha512 ( szTest_B, strlen(szTest_B), sHash );
    sha2_test_hash2ascii ( sHash, szHashAscii, SHA512_HASHSIZE );
    if (strcmp(szHashAscii,SHA_512_B)==0) printf ( "OK\n" );
    else printf ( "Error : %s\n", szHashAscii ); 

    printf ( "Test C 'Long block message' :\n" );
    memset ( sHash, 0, 256 );
    sha512 ( szTest_C, strlen(szTest_C), sHash );
    sha2_test_hash2ascii ( sHash, szHashAscii, SHA512_HASHSIZE );
    if (strcmp(szHashAscii,SHA_512_C)==0) printf ( "OK\n" );
    else printf ( "Error : %s\n", szHashAscii ); 
    
    free ( szTest_C );
}

#endif

